/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.engine;

// IMPORTS
import javax.microedition.broadcast.purchase.PriceOption;
import javax.microedition.broadcast.purchase.PurchaseObject;
import javax.microedition.broadcast.purchase.SubscriptionManager;

/**
 * Controls subscriptions.
 */
public class SubscriptionController {
    // Singleton instance

    private static SubscriptionController subscriptionController;
    // ID used for getting Purchase Objects.
    private String purchaseObjectId;
    // Selected purchase object.
    private PurchaseObject selectedObject;

    /**
     * Returns the sigleton instance of this class
     */
    public static SubscriptionController getInstance() {
        if (subscriptionController == null) {
            subscriptionController = new SubscriptionController();
        }

        return subscriptionController;
    }

    /**
     * Sets ID of program or service to be used when
     * getPurchaseObjects() is called.
     * Setting null will return all purchase objects.
     */
    public void setId(String argId) {
        purchaseObjectId = argId;
    }

    /**
     * Returns purchase objects matching the ID
     * set by setId().
     */
    public PurchaseObject[] getPurchaseObjects() {
        PurchaseObject[] varPurchaseOjects;

        if (purchaseObjectId == null) {
            varPurchaseOjects = SubscriptionManager.getAllPurchaseObjects();
        } else {
            varPurchaseOjects =
                    SubscriptionManager.getPurchaseObjects(purchaseObjectId);
        }

        return varPurchaseOjects;
    }

    /**
     * Returns the default purchase channel.
     */
    public String getDefaultChannel() {
        return SubscriptionManager.getDefaultPurchaseChannel();
    }

    /**
     * Returns all supported purchase channels.
     */
    public String[] getSupportedChannels() {
        return SubscriptionManager.getSupportedPurchaseChannels();
    }

    /**
     * Selects a purchase object. This object will be used when price options
     * are asked or the object is purchased.
     */
    public void selectPurchaseObject(PurchaseObject argPurchaseObject) {
        selectedObject = argPurchaseObject;
    }

    /**
     * Returns the selected purchase object or null if no objects are
     * selected.
     */
    public PurchaseObject getSelectedPurchaseObject() {
        return selectedObject;
    }

    /**
     * Returns available price options for the selected purchase object.
     */
    public PriceOption[] getPriceOptions() {
        return selectedObject.getPriceOptions();
    }

    /**
     * Purchases the selected purchase object by using the given
     * price option.
     */
    public void purchase(PriceOption argOption) {
        selectedObject.purchase(argOption);
    }

    /**
     * Requests price options from the selected purchase object.
     */
    public void requestPriceOptions() {
        selectedObject.requestPriceOptions();
    }

    /**
     * Returns all subscribed purchase objects.
     */
    public PurchaseObject[] getSubscriptions() {
        return SubscriptionManager.getSubscriptions();
    }

    /**
     * Unsubscribes the selected purchase object.
     */
    public void unsubscribe() {
        selectedObject.cancel();
    }

    private SubscriptionController() {
    }
}
