/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
*/

// PACKAGE
package com.nokia.example.ui;

// EXTERNAL IMPORTS
import java.util.Vector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.broadcast.esg.Attribute;
import javax.microedition.broadcast.esg.CommonMetadataSet;

/**
 * Displays a list of available attributes to be used for sorting ESG data
 */
public class AddSortAttributes extends List
{
    // Next displayable
    private Displayable nextDisplayable;
    // Vector of currently selected attributes
    private Vector selectedAttributes;
    // Vector of currently available attributes
    private Vector availableAttributes;

    /**
     * Constructor
     */
    public AddSortAttributes(
        String argTitle,
        Displayable argNextDisplayable,
        Vector argSelectedAttributes)
    {
        super(argTitle, List.EXCLUSIVE);

        nextDisplayable = argNextDisplayable;
        selectedAttributes = argSelectedAttributes;
        reset();
    }

    /**
     * Activates this view as the current one
     */
    public void activate()
    {
        Display varDisplay = ViewHandler.getDisplay();
        if (varDisplay != null)
        {
            varDisplay.setCurrent(this);
            refreshList();
        }
    }

    /**
     * Deactivates this view and switches to the next one
     */
    public void deactivate()
    {
        Display varDisplay = ViewHandler.getDisplay();
        if (varDisplay != null)
        {
            varDisplay.setCurrent(nextDisplayable);
        }
    }

    /**
     * Adds a new attribute to the selected list from available ones
     */
    public void add()
    {
        int varIndex = getSelectedIndex();
        if (varIndex > -1 && varIndex < availableAttributes.size() - 1)
        {
            Attribute varAttribute =
                (Attribute)availableAttributes.elementAt(varIndex);
            selectedAttributes.addElement(varAttribute);
            availableAttributes.removeElement(varAttribute);
            refreshList();
        }
    }

    /**
     * Resets both selected and available attribute vectors
     */
    public void reset()
    {
        selectedAttributes.removeAllElements();
        availableAttributes = getCommonAttributes();
    }

    /**
     * Returns a vector of available CommonMetaDataSet attributes
     *
     * @return Vector of found attributes
     */
    private Vector getCommonAttributes()
    {
        Vector varAvailable = new Vector();

        CommonMetadataSet metadata = CommonMetadataSet.getInstance();
        if (metadata  != null)
        {
            Attribute[] varAttributes = metadata.getValidAttributes();

            // Add available CommonMetadataSet attributes
            int varLength = varAttributes.length;
            for (int varIndex = 0; varIndex < varLength; ++varIndex)
            {
                varAvailable.addElement(varAttributes[ varIndex ]);
            }
        }

        return varAvailable;
    }

    /**
     * Refreshes the available attribute item list on the display
     */
    private void refreshList()
    {
        deleteAll();
        int varSize = availableAttributes.size();
        for (int varIndex = 0; varIndex < varSize; ++varIndex)
        {
            Attribute varAttribute =
                    (Attribute)availableAttributes.elementAt(varIndex);
            append(varAttribute.getName(), null);
            setFont(varIndex, ViewUtils.getDefaultFont());
        }
    }
}

// End of file
