/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.ServiceGuideController;

// EXTERNAL IMPORTS
import java.util.Date;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.TextField;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.StringItem;
import javax.microedition.broadcast.platform.PlatformProvider;
import javax.microedition.broadcast.platform.PlatformProviderSelector;
import javax.microedition.broadcast.esg.ServiceGuideListener;
import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.broadcast.BroadcastServiceException;

public class PlatformProviderView extends Form
        implements ViewBase, ServiceGuideListener {
    // Current PlatformProvider

    private PlatformProvider provider;
    private ServiceGuideController controller;
    //private SubscriptionController iSubscriptionController;
    // String items
    private StringItem description;
    private StringItem providerId;
    private StringItem esgUpdateTime;
    private StringItem networkTime;
    // Commands
    private Command cmdFindPrograms;
    private Command cmdFindServices;
    private Command cmdComposeQuery;
    private Command cmdSelectSortAttributes;
    private Command cmdUpdateESG;
    private Command cmdListenESG;
    private Command cmdStopListeningESG;
    private Command cmdManageSubscriptions;
    private Command cmdBack;

    /**
     * Constructor
     */
    PlatformProviderView() {
        super("Current Platform Provider");

        cmdFindPrograms = new Command("List programs", Command.SCREEN, 2);
        cmdFindServices = new Command("List services", Command.SCREEN, 2);
        cmdComposeQuery = new Command("Define a query", Command.SCREEN, 2);
        cmdSelectSortAttributes =
                new Command("Select sort attributes", Command.SCREEN, 2);
        cmdUpdateESG = new Command("Force ESG update", Command.SCREEN, 2);
        cmdListenESG = new Command("Listen ESG", Command.SCREEN, 2);
        cmdStopListeningESG =
                new Command("Stop listening ESG", Command.SCREEN, 2);
        cmdManageSubscriptions =
                new Command("Manage subscriptions", Command.SCREEN, 2);
        cmdBack = new Command("Back", Command.BACK, 2);

        description = new StringItem("Description:", "");
        description.setFont(ViewUtils.getDefaultFont());

        providerId = new StringItem("Identifier:", "");
        providerId.setFont(ViewUtils.getDefaultFont());

        esgUpdateTime = new StringItem("ESG update time:", "");
        esgUpdateTime.setFont(ViewUtils.getDefaultFont());

        networkTime = new StringItem("Network time:", "");
        networkTime.setFont(ViewUtils.getDefaultFont());

        addCommand(cmdFindPrograms);
        addCommand(cmdFindServices);
        addCommand(cmdComposeQuery);
        addCommand(cmdSelectSortAttributes);
        addCommand(cmdUpdateESG);
        addCommand(cmdListenESG);
        addCommand(cmdStopListeningESG);
        addCommand(cmdManageSubscriptions);
        addCommand(cmdBack);

        append(description);
        append(providerId);
        append(esgUpdateTime);
        append(networkTime);
    }

    /**
     * From ServiceGuideListener
     */
    public void serviceGuideUpdated(
            String argEvent,
            ServiceGuideData argDerviceGuideData,
            Object argEventData) {
        AlertNote varNote = null;
        ViewBase varCurrentView = ViewHandler.getCurrentView();
        Displayable varDisplayable = varCurrentView.getDisplayable();

        System.out.println(
                "PlatformProviderView.serviceGuideUpdated() aEvent=" +
                argEvent);

        if (argEvent.equalsIgnoreCase(
                ServiceGuideListener.SERVICE_GUIDE_UPDATE_STARTED)) {
            varNote = new AlertNote("Service guide change",
                    "ESG update started",
                    AlertType.INFO,
                    varDisplayable);
        } else if (argEvent.equalsIgnoreCase(
                ServiceGuideListener.SERVICE_GUIDE_UPDATE_COMPLETED)) {
            varNote = new AlertNote("Service guide change",
                    "ESG update completed",
                    AlertType.INFO,
                    varDisplayable);
        } else if (argEvent.equalsIgnoreCase(
                ServiceGuideListener.SERVICE_GUIDE_UPDATE_FAILED)) {
            varNote = new AlertNote("Service guide change",
                    "ESG update failed!",
                    AlertType.ERROR,
                    varDisplayable);
        } else if (argEvent.equalsIgnoreCase(
                ServiceGuideListener.SERVICE_GUIDE_DELETED)) {
            varNote = new AlertNote("Service guide change",
                    "ESG invalidated",
                    AlertType.INFO,
                    varDisplayable);
            varNote.execute();
            return;
        }

        if (varNote != null) {
            varNote.execute();
        }

        updateProviderInfo();
    }

    /**
     * Activates the view
     */
    public void activate() {
        // Get currently selected platform provider
        provider = PlatformProviderSelector.getCurrentProvider();
        // Initialize Service guide
        controller = ServiceGuideController.getInstance();
        controller.initialize();

        updateProviderInfo();
    }

    /**
     * Deactivates the view
     */
    public void deactivate() {
        // Nothing to do here
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable() {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param aListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener argListener) {
        super.setCommandListener(argListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param argCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable) {
        if (argCommand == cmdFindPrograms) {
            controller.setService(null);
            controller.resetSortKeys();
            // No queries currently used
            controller.setQuery(null, ServiceGuideController.QUERY_PROGRAMS);

            // Program view initializes and gets the data from the guide
            ViewHandler.activateView(ViewHandler.PROGRAMS_VIEW_ID);
        } else if (argCommand == cmdFindServices) {
            controller.resetSortKeys();
            // No queries currently used
            controller.setQuery(null, ServiceGuideController.QUERY_SERVICES);

            // Service view initializes and gets the data from the guide
            ViewHandler.activateView(ViewHandler.SERVICES_VIEW_ID);
        } else if (argCommand == cmdUpdateESG) {
            controller.updateServiceGuide();
        } else if (argCommand == cmdComposeQuery) {
            ViewHandler.activateView(ViewHandler.QUERY_COMPOSER_VIEW_ID);
        } else if (argCommand == cmdSelectSortAttributes) {
            ViewHandler.activateView(ViewHandler.SORT_ATTRIBUTES_VIEW_ID);
        } else if (argCommand == cmdBack) {
            ViewHandler.activateView(ViewHandler.PLATFORM_VIEW_ID);
        } else if (argCommand == cmdListenESG) {
            controller.addListener(this);
        } else if (argCommand == cmdStopListeningESG) {
            controller.removeListener(this);
        } else if (argCommand == cmdManageSubscriptions) {
            ViewHandler.activateView(ViewHandler.SUBSCRIPTION_VIEW_ID);
        }
    }

    /**
     * Updates the provider information
     */
    private void updateProviderInfo() {
        description.setText(provider.toString());
        providerId.setText(Long.toString(provider.getID()));

        try {
            esgUpdateTime.setText(controller.getUpdateTime().toString());
        } catch (Exception e) {
            esgUpdateTime.setText("Not available");
        }
        try {
            networkTime.setText(
                    PlatformProviderSelector.getCurrentProviderDate().
                    toString());
        } catch (Exception e) {
            networkTime.setText("Not available");
        }
    }
}

// End of File

