/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.SubscriptionController;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.broadcast.purchase.PriceOption;
import javax.microedition.broadcast.purchase.PurchaseListener;
import javax.microedition.broadcast.purchase.PurchaseObject;
import java.util.Vector;

/**
 * Shows information about service components.
 */
public class PriceOptionView extends List
        implements ViewBase, PurchaseListener {

    SubscriptionController controller;
    // Available price options.
    Vector priceOptions;
    // Commands
    private Command cmdRequestOptions;
    private Command cmdPurchase;
    private Command cmdBack;

    /**
     * Constructor
     */
    PriceOptionView() {
        super("Price options", List.IMPLICIT);

        controller = SubscriptionController.getInstance();
        priceOptions = new Vector(2);

        cmdRequestOptions =
                new Command("Request price options", Command.SCREEN, 1);
        cmdPurchase = new Command("Purchase", Command.SCREEN, 1);
        cmdBack = new Command("Back", Command.BACK, 1);

        addCommand(cmdRequestOptions);
        setSelectCommand(cmdPurchase);
        addCommand(cmdBack);
    }

    /**
     * Activates the view
     */
    public void activate() {
        reloadPriceOptions();
    }

    /**
     * Deactivates the view
     */
    public void deactivate() {
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable() {
        return this;
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param aCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command aCommand, Displayable argDisplayable) {
        if (aCommand == cmdBack) {
            ViewHandler.activatePreviousView();
            return;
        }


        PriceOption varOption = null;

        synchronized (priceOptions) {
            int varindex = getSelectedIndex();
            if (varindex >= 0 && varindex < priceOptions.size()) {
                varOption = (PriceOption) priceOptions.elementAt(varindex);
            }
        }

        if (varOption == null) {
            // Nothing selected.
            return;
        }

        if (aCommand == cmdPurchase) {
            purchase(varOption);
        } else if (aCommand == cmdRequestOptions) {
            requestPriceOptions();
        }
    }

    /**
     * From PurchaseListener.
     */
    public void purchaseResponse(PurchaseObject argSource,
            String argEvent,
            Object argEventData) {
        System.out.println("PriceOptionView.purchaseResponse: " + argEvent);

        ViewBase varCurrentView = ViewHandler.getCurrentView();
        Displayable varCurrent = varCurrentView.getDisplayable();

        if (!argEvent.startsWith("Cancel")) {
            String varMessage = argEvent;
            if (argEventData instanceof String) {
                varMessage += ", " + argEventData;
                System.out.println("PriceOptionView.purchaseResponse, data="
                        + argEventData);
            }

            AlertNote varAlertNote = new AlertNote("Purchase response",
                    varMessage,
                    AlertType.INFO,
                    varCurrent);
            varAlertNote.execute();
        }

        if (argEvent.equalsIgnoreCase(PRICE_INFO_RECEIVED)
                || argEvent.equalsIgnoreCase(PRICE_INFO_RECEIVAL_FAILED)) {
            reloadPriceOptions();
        }
    }

    /**
     * Asks the price options from the controller and appends
     * them to the list.
     */
    private void reloadPriceOptions() {
        PriceOption[] varPriceOptions = controller.getPriceOptions();

        synchronized (priceOptions) {
            deleteAll();
            priceOptions.removeAllElements();

            for (int varIndex = 0;
                    varIndex < varPriceOptions.length; varIndex++) {
                priceOptions.addElement(varPriceOptions[varIndex]);

                String varTemp = varPriceOptions[varIndex].getPrice() + " ("
                        + varPriceOptions[varIndex].getPriceDescription() + ")";

                append(varTemp, null);
            }
        }
    }

    /**
     * Starts to purchase the selected purchase object with
     * the given price option.
     */
    private void purchase(PriceOption argPriceOption) {
        PurchaseObject varPurchaseObject =
                controller.getSelectedPurchaseObject();
        varPurchaseObject.addPurchaseListener(this);
        controller.purchase(argPriceOption);
    }

    /**
     * Request price options for the selected purchase object.
     */
    private void requestPriceOptions() {
        PurchaseObject varPurchaseObject =
                controller.getSelectedPurchaseObject();
        varPurchaseObject.addPurchaseListener(this);
        controller.requestPriceOptions();
    }
}

// End of File

