/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
*/

// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.SubscriptionController;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.broadcast.purchase.PurchaseObject;

/**
 * Displays information from the specified purchase object.
 */
public class PurchaseObjectDataInfo extends ServiceGuideDataInfo
{
    // Available channels. The first index means no channel.
    private ChoiceGroup channels;

    /**
     * Constructor
     */
    public PurchaseObjectDataInfo(
        String argTitle,
        Displayable argNextDisplayable,
        ServiceGuideData argData)
    {
        super(argTitle, argNextDisplayable, argData);
    }

    /**
     * Derived from ServiceGuideDataInfo.
     */
    public void commandAction(Command argCommand, Displayable argDisplayable)
    {
        if (argCommand == cmdBack)
        {
            try
            {
                selectChannel();
            }
            catch (Throwable t)
            {
                t.printStackTrace();
            }
            super.commandAction(argCommand, argDisplayable);
        }
    }

    /**
     * Derived from ServiceGuideDataInfo.
     */
    protected void reloadMetadata()
    {
        if (channels == null)
        {
            channels =
                new ChoiceGroup("Purchase channel", ChoiceGroup.EXCLUSIVE);
        }

        deleteAll();

        loadChannelData();
        append(channels);

        refreshMetadata();
    }

    /**
     * Loads purchase channel data to the choice group.
     */
    private void loadChannelData()
    {
        channels.deleteAll();

        // The first index means no channel.
        channels.append("No channel used", null);

        // Add supported channels to the list.
        SubscriptionController varController =
                SubscriptionController.getInstance();
        String[] varSupportedChannels = varController.getSupportedChannels();

        for (int varIndex = 0;
                varIndex < varSupportedChannels.length; varIndex++)
        {
            channels.append(varSupportedChannels[ varIndex ], null);
        }

        // Select the channel that is returned by the object.
        PurchaseObject varObject = (PurchaseObject)sgData;
        String varChannel = varObject.getPurchaseChannel();

        for (int varIndex = 0;
                varIndex < varSupportedChannels.length; varIndex++)
        {
            if (varSupportedChannels[ varIndex ].equals(varChannel))
            {
                channels.setSelectedIndex(varIndex + 1, true);
            }
        }
    }

    /**
     * Stores the selected channel to the purchase object.
     * If no channel is selected, the choice is ignored.
     */
    private void selectChannel()
    {
        int varSelected = channels.getSelectedIndex();

        // Do not made any changes if the first index (no channel)
        // is selected. This is because the user cannot remove purchase
        // channel from the purchase object but one should be used.
        if (varSelected > 0)
        {
            String varChannel = channels.getString(varSelected);

            PurchaseObject varObject = (PurchaseObject)sgData;
            varObject.setPurchaseChannel(varChannel);
        }

    }
}

// End of File
