/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
*/

// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.ServiceGuideController;
import com.nokia.example.engine.SubscriptionController;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.broadcast.ServiceComponent;
import javax.microedition.broadcast.esg.CommonMetadataSet;
import javax.microedition.broadcast.esg.DataUnavailableException;
import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.broadcast.esg.ServiceGuideListener;
import javax.microedition.broadcast.purchase.PurchaseListener;
import javax.microedition.broadcast.purchase.PurchaseObject;
import java.io.IOException;
import java.util.Vector;

/**
 * Shows information about purchase objects.
 */
public class PurchaseObjectsView extends List
        implements ViewBase, ServiceGuideListener, PurchaseListener
{
    // Constants
    protected static final String PURCHASED_IMAGE = "/res/purchased.gif";
    protected static final String NOTPURCHASED_IMAGE = "/res/notpurchased.gif";

    // Commands
    private Command cmdBack;
    private Command cmdShowInfo;
    private Command cmdListPriceOptions;
    private Command cmdUnsubscribe;

    protected SubscriptionController controller;
    protected ServiceGuideController esgController;
    protected Vector purchaseObjects;

    /**
     * Constructor
     */
    PurchaseObjectsView()
    {
        super("Purchase objects", List.IMPLICIT);

        purchaseObjects = new Vector(5);
        controller = SubscriptionController.getInstance();

        esgController = ServiceGuideController.getInstance();

        cmdBack = new Command("Back", Command.BACK, 1);
        cmdShowInfo = new Command("Show details", Command.SCREEN, 1);
        cmdListPriceOptions =
                new Command("List price options", Command.SCREEN, 1);
        cmdUnsubscribe = new Command("Unsubscribe", Command.SCREEN, 1);

        addCommand(cmdShowInfo);
        setSelectCommand(cmdListPriceOptions);
        addCommand(cmdUnsubscribe);
        addCommand(cmdBack);
    }

    /**
     * From ServiceGuideListener
     */
    public void serviceGuideUpdated(
        String argEvent,
        ServiceGuideData argServiceGuideData,
        Object argEventData)
    {
        // Update the list if ESG has updated.
        if (argEvent.equalsIgnoreCase(
                    ServiceGuideListener.SERVICE_GUIDE_BULK_CHANGED) ||
                argEvent.equalsIgnoreCase(
                    ServiceGuideListener.PURCHASE_OBJECT_ADDED) ||
                argEvent.equalsIgnoreCase(
                    ServiceGuideListener.PURCHASE_OBJECT_DELETED))
        {
            reloadPurchaseObjects();
        }
    }

    /**
     * From PurchaseListener.
     */
    public void purchaseResponse(PurchaseObject argSource,
                                 String argEvent,
                                 Object argEventData)
    {
        System.out.println("PurchaseObjectsView.purchaseResponse: " + argEvent);

        ViewBase varCurrentView = ViewHandler.getCurrentView();
        Displayable varCurrent = varCurrentView.getDisplayable();


        if (argEvent.startsWith("Cancel"))
        {
            String varMessage = argEvent;
            if (argEventData instanceof String)
            {
                varMessage += ", " + argEventData;
                System.out.println(
                        "PurchaseObjectsView.purchaseResponse, data=" +
                                   argEventData);
            }

            AlertNote varAlertNote = new AlertNote("Cancel response",
                                           varMessage,
                                           AlertType.INFO,
                                           varCurrent);
            varAlertNote.execute();
        }

        if (argEvent.equalsIgnoreCase(PurchaseListener.CANCEL_COMPLETED) ||
                argEvent.equalsIgnoreCase(PurchaseListener.PURCHASE_COMPLETED))
        {
            reloadPurchaseObjects();
        }
    }

    /**
     * Activates the view
     */
    public void activate()
    {
        esgController.addListener(this);
        reloadPurchaseObjects();
    }

    /**
     * Deactivates the view
     */
    public void deactivate()
    {
        deleteAll();
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable()
    {
        return this;
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param aCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable)
    {
        if (argCommand == cmdBack)
        {
            ViewHandler.activatePreviousView();
            return;
        }

        int varIndex = getSelectedIndex();
        if (varIndex < 0)
        {
            // Nothing to do.
            return;
        }

        PurchaseObject varObject =
            (PurchaseObject)purchaseObjects.elementAt(varIndex);

        if (argCommand == cmdShowInfo)
        {
            new PurchaseObjectDataInfo(getString(varIndex), this, varObject);
        }
        else if (argCommand == cmdListPriceOptions)
        {
            varObject.addPurchaseListener(this);
            controller.selectPurchaseObject(varObject);
            ViewHandler.activateView(ViewHandler.PRICE_OPTION_VIEW_ID);
        }
        else if (argCommand == cmdUnsubscribe)
        {
            varObject.addPurchaseListener(this);
            controller.selectPurchaseObject(varObject);
            controller.unsubscribe();
        }
    }

    /**
     * Asks purchase objects from the engine and updates the list.
     */
    protected synchronized void reloadPurchaseObjects()
    {
        Image varPurchasedImage = null;
        Image varNotPurchasedImage = null;

        Class varTempClass = getClass();

        try
        {
            varPurchasedImage = Image.createImage(
                        varTempClass.getResourceAsStream(PURCHASED_IMAGE));

            varNotPurchasedImage = Image.createImage(
                        varTempClass.getResourceAsStream(NOTPURCHASED_IMAGE));
        }
        catch (IOException ioe)
        {
            // Ignored, no icons shown for the user.
            ioe.printStackTrace();
        }

        deleteAll();
        purchaseObjects.removeAllElements();

        PurchaseObject[] varObjects = controller.getPurchaseObjects();

        for (int varIndex = 0; varIndex < varObjects.length; varIndex++)
        {
            String varName;

            try
            {
                varName = (String)varObjects[ varIndex ].getStringValue(
                           CommonMetadataSet.PURCHASE_ITEM_NAME);
            }
            catch (DataUnavailableException due)
            {
                varName = "(no name)";
            }

            Image varImage = (varObjects[ varIndex ].isPurchased() ? 
                        varPurchasedImage : varNotPurchasedImage);

            purchaseObjects.addElement(varObjects[ varIndex ]);
            append(varName, varImage);
        }
    }
}

// End of File
