/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.PresentationController;

// EXTERNAL IMPORTS
import java.util.Vector;
import javax.microedition.broadcast.ServiceComponent;
import javax.microedition.broadcast.esg.CommonMetadataSet;
import javax.microedition.broadcast.esg.DataUnavailableException;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.Displayable;

/**
 * Shows information about service components.
 */
public class ServiceComponentsView extends List implements ViewBase {
    // Commands

    private Command CmdBack;
    private PresentationController controller;
    private Vector serviceComponents;
    private Command cmdInfo;

    /**
     * Constructor
     */
    ServiceComponentsView() {
        super("Service components", List.IMPLICIT);

        CmdBack = new Command("Back", Command.BACK, 1);
        cmdInfo = new Command("Show details", Command.SCREEN, 1);
        addCommand(CmdBack);
        addCommand(cmdInfo);

        controller = PresentationController.getInstance();
        serviceComponents = new Vector();
    }

    /**
     * Activates the view
     */
    public void activate() {
        serviceComponents.removeAllElements();
        deleteAll();
        controller = PresentationController.getInstance();

        ServiceComponent[] varServiceComponents = controller.getAllComponents();
        for (int varIndex = 0;
                varIndex < varServiceComponents.length; varIndex++) {
            addServiceComponent(varServiceComponents[varIndex]);
        }
    }

    /**
     * Adds new service component to the list
     */
    public synchronized void addServiceComponent(
                    ServiceComponent argServiceComponent) {
        serviceComponents.addElement(argServiceComponent);
        try {
            append(argServiceComponent.getStringValue(
                    CommonMetadataSet.SERVICE_COMPONENT_SDP_REF), null);
        } catch (DataUnavailableException due) {
            append("(Unnamed)", null);
        }

        int varIndex = serviceComponents.size() - 1;
        setFont(varIndex, ViewUtils.getDefaultFont());
    }

    /**
     * Deactivates the view
     */
    public void deactivate() {
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable() {
        return this;
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param aCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable) {
        if (argCommand == CmdBack) {
            ViewHandler.activatePreviousView();
        }

        int varIndex = getSelectedIndex();
        if (varIndex < 0) {
            return;
        }

        ServiceComponent varServicecomponent = (ServiceComponent)
                        serviceComponents.elementAt(varIndex);

        if (argCommand == cmdInfo) {
            new ServiceGuideDataInfo(getString(varIndex),
                    this, varServicecomponent);
        }
    }
}

// End of File

