/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
*/

// PACKAGE
package com.nokia.example.ui;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;

/**
 * Start view of the application
 */
public class MainView extends Form implements ViewBase
{
    // Commands
    private Command cmdExit;
    private Command cmdStart;

    /**
     * Constructor
     */
    MainView()
    {
        super("JSR-272 Test MIDlet");

        cmdExit = new Command("Exit", Command.EXIT, 1);
        cmdStart = new Command("Select platform", Command.SCREEN, 2);

        addCommand(cmdExit);
        addCommand(cmdStart);
    }

    /**
     * Activates the view
     */
    public void activate()
    {
        append("JSR-272 MBS API functional testing MIDlet\n");
        append("\nAPI version is: " +
                System.getProperty("microedition.broadcast.version"));
        append("\nSupports recording: " + System.getProperty(
                "microedition.broadcast.supports.timedrecording"));
        append("\nSupports file cache: " + System.getProperty(
                "microedition.broadcast.supports.filecache"));
        append("\nSupports overlay: " +
                System.getProperty("microedition.broadcast.supports.overlay"));
        append("\nSupports purchasing: " + System.getProperty(
                "microedition.broadcast.supports.purchasing"));
    }

    /**
     * Deactivates the view
     */
    public void deactivate()
    {
        deleteAll();
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable()
    {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param argListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener argListener)
    {
        super.setCommandListener(argListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param argCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable)
    {
        if (argCommand == cmdStart)
        {
            ViewHandler.activateView(ViewHandler.PLATFORM_VIEW_ID);
        }
    }
}

// End of File
