/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.ServiceGuideController;

// EXTERNAL IMPORTS
import java.util.Date;
import java.util.Vector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.AlertType;
import javax.microedition.broadcast.esg.Query;
import javax.microedition.broadcast.esg.QueryComposer;
import javax.microedition.broadcast.esg.CommonMetadataSet;

public class QueryComposerView extends List implements ViewBase {
    // CONSTANTS

    private final static int SET = 0;
    private final static int AND = 1;
    private final static int OR = 2;
    private final static int NOT = 3;
    // MEMBERS
    private QueryData currentQuery;
    private Vector equivalentQueries;
    private ServiceGuideController serviceGuideController;
    // Commands
    private Command cmdShowQuery;
    private Command cmdReset;
    private Command cmdFindPrograms;
    private Command cmdFindServices;
    private Command cmdSetCurrent;
    private Command cmdNot;
    private Command cmdOr;
    private Command cmdAnd;
    private Command cmdBack;

    /**
     * Utility class for storing ESG query objects and additional information
     */
    private class QueryData {
        // Text to be displayed in the list

        public String text;
        // Freeform ID for the query
        public String queryId;
        // ESG Query object
        public Query query;

        /**
         * Constructor
         */
        QueryData(String argText, String argId, Query argQuery) {
            text = argText;
            queryId = argId;
            query = argQuery;
        }
    }

    /**
     * Constructor
     */
    QueryComposerView() {
        super("Query Composer", List.MULTIPLE);

        cmdShowQuery = new Command("Show current query", Command.SCREEN, 10);
        cmdReset = new Command("Reset current query", Command.SCREEN, 9);
        cmdFindPrograms = new Command("Find programs", Command.SCREEN, 8);
        cmdFindServices = new Command("Find services", Command.SCREEN, 7);
        cmdSetCurrent = new Command("Set as current", Command.SCREEN, 6);
        cmdNot = new Command("Not", Command.SCREEN, 5);
        cmdOr = new Command("Or", Command.SCREEN, 4);
        cmdAnd = new Command("And", Command.SCREEN, 3);
        cmdBack = new Command("Back", Command.BACK, 2);

        serviceGuideController = ServiceGuideController.getInstance();

        // Add few hard coded queries which can be used as building blocks
        // for more complex ones
        equivalentQueries = new Vector();
        try {
            equivalentQueries.addElement(
                    new QueryData("currentProgram", "0",
                    QueryComposer.currentProgram()));
            equivalentQueries.addElement(
                    new QueryData("SERVICE_GENRE = Movie", "1",
                    QueryComposer.equivalent(
                    CommonMetadataSet.SERVICE_GENRE, "Movie")));
            equivalentQueries.addElement(
                    new QueryData("SERVICE_GENRE = Education", "2",
                    QueryComposer.equivalent(
                    CommonMetadataSet.SERVICE_GENRE, "Education")));
            equivalentQueries.addElement(
                    new QueryData("SERVICE_GENRE = News", "3",
                    QueryComposer.equivalent(
                    CommonMetadataSet.SERVICE_GENRE, "News")));
            equivalentQueries.addElement(
                    new QueryData("SERVICE_GENRE = Sports", "4",
                    QueryComposer.equivalent(
                    CommonMetadataSet.SERVICE_GENRE, "Sports")));
            equivalentQueries.addElement(
                    new QueryData("SERVICE_NAME contains Mtv", "5",
                    QueryComposer.contains(
                    CommonMetadataSet.SERVICE_NAME, "Mtv")));
            equivalentQueries.addElement(
                    new QueryData("SERVICE_NAME contains Voice", "6",
                    QueryComposer.contains(
                    CommonMetadataSet.SERVICE_NAME, "Voice")));
            equivalentQueries.addElement(
                    new QueryData("PROGRAM_START_TIME > now", "7",
                    QueryComposer.after(
                    CommonMetadataSet.PROGRAM_START_TIME, new Date())));
            equivalentQueries.addElement(
                    new QueryData("PROGRAM_START_TIME < now", "8",
                    QueryComposer.before(
                    CommonMetadataSet.PROGRAM_START_TIME, new Date())));
            equivalentQueries.addElement(
                    new QueryData("PROGRAM_END_TIME > now", "9",
                    QueryComposer.after(
                    CommonMetadataSet.PROGRAM_END_TIME, new Date())));
            equivalentQueries.addElement(
                    new QueryData("PROGRAM_END_TIME < now", "10",
                    QueryComposer.before(
                    CommonMetadataSet.PROGRAM_END_TIME, new Date())));
            equivalentQueries.addElement(
                    new QueryData("PROGRAM_CONTENT_GENRE = Arts", "11",
                    QueryComposer.equivalent(
                    CommonMetadataSet.PROGRAM_CONTENT_GENRE, "Arts")));
            equivalentQueries.addElement(
                    new QueryData("PROGRAM_CONTENT_GENRE = News", "12",
                    QueryComposer.equivalent(
                    CommonMetadataSet.PROGRAM_CONTENT_GENRE, "News")));
            equivalentQueries.addElement(
                    new QueryData("PROGRAM_CONTENT_GENRE = Sports", "13",
                    QueryComposer.equivalent(
                    CommonMetadataSet.PROGRAM_CONTENT_GENRE, "Sports")));
            equivalentQueries.addElement(
                    new QueryData("PROGRAM_CONTENT_GENRE = Music", "14",
                    QueryComposer.equivalent(
                    CommonMetadataSet.PROGRAM_CONTENT_GENRE, "Music")));
        } catch (Exception e) {
            e.printStackTrace();
            AlertNote varAlert = new AlertNote("Exception",
                    e.toString(),
                    AlertType.ERROR,
                    this);
            varAlert.execute();
        }

        // Add list items
        int varSize = equivalentQueries.size();
        for (int varIndex = 0; varIndex < varSize; varIndex++) {
            QueryData varQuery = (QueryData)
                    equivalentQueries.elementAt(varIndex);
            append(varQuery.text, null);
            setFont(varIndex, ViewUtils.getDefaultFont());
        }
    }

    /**
     * Activates the view
     */
    public void activate() {
        addCommand(cmdBack);
        addCommand(cmdAnd);
        addCommand(cmdOr);
        addCommand(cmdNot);
        addCommand(cmdSetCurrent);
        addCommand(cmdFindServices);
        addCommand(cmdFindPrograms);
        addCommand(cmdReset);
        addCommand(cmdShowQuery);
    }

    /**
     * Deactivates the view
     */
    public void deactivate() {
        removeCommand(cmdShowQuery);
        removeCommand(cmdReset);
        removeCommand(cmdFindPrograms);
        removeCommand(cmdFindServices);
        removeCommand(cmdSetCurrent);
        removeCommand(cmdNot);
        removeCommand(cmdOr);
        removeCommand(cmdAnd);
        removeCommand(cmdBack);
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable() {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param aListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener argListener) {
        super.setCommandListener(argListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param argCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable) {
        boolean varShow = false;
        AlertNote varAlert = null;
        if (argCommand == cmdBack) {
            currentQuery = null;
            ViewHandler.activateView(ViewHandler.PLATFORM_PROVIDER_VIEW_ID);
        } else if (argCommand == cmdReset) {
            currentQuery = null;
            deselectAllItems();
            (new AlertNote("Query reseted",
                    "",
                    AlertType.INFO,
                    this)).execute();
        } else if (argCommand == cmdAnd) {
            currentQuery = doQueries(AND, currentQuery, getSelectedItems());
            deselectAllItems();
            varShow = true;
        } else if (argCommand == cmdOr) {
            currentQuery = doQueries(OR, currentQuery, getSelectedItems());
            deselectAllItems();
            varShow = true;
        } else if (argCommand == cmdNot) {
            currentQuery = doQueries(NOT, currentQuery, new Vector());
            deselectAllItems();
            varShow = true;
        } else if (argCommand == cmdSetCurrent) {
            currentQuery = doQueries(SET, currentQuery, getSelectedItems());
            deselectAllItems();
            varShow = true;
        } else if (argCommand == cmdFindPrograms) {
            serviceGuideController.setQuery(
                    currentQuery.query,
                    ServiceGuideController.QUERY_PROGRAMS);
            ViewHandler.activateView(ViewHandler.PROGRAMS_VIEW_ID);
        } else if (argCommand == cmdFindServices) {
            serviceGuideController.setQuery(
                    currentQuery.query,
                    ServiceGuideController.QUERY_SERVICES);
            ViewHandler.activateView(ViewHandler.SERVICES_VIEW_ID);
        } else if (argCommand == cmdShowQuery) {
            varShow = true;
        }

        if (currentQuery != null && varShow) {
            varAlert = new AlertNote("Current query",
                    currentQuery.queryId,
                    AlertType.INFO,
                    this);
            varAlert.execute();
        }
    }

    /**
     * Executes sequence of queries of requested query type ie. SET, AND, OR
     * or NOT.
     *
     * @param aQueryType Operation enumeration type that is one of the
     *        SET, AND, OR or NOT ones.
     * @param argData Result of possible previous query execution which will
     *        used as a starting point (if defined) before processing aQueries.
     * @param argQueries Vector of queries to be executed
     */
    private QueryData doQueries(
            int argQueryType,
            QueryData argData,
            Vector argQueries) {
        QueryData varQueryData = argData;
        if (varQueryData == null) {
            // There was no previous query so set first one from the aQueries
            // as the starting point, if applicaple
            varQueryData =
                    argQueries.size() > 0 ?
                        (QueryData) argQueries.elementAt(0) : null;
        } else {
            // There is a previous query, insert it to the beginning
            argQueries.insertElementAt(varQueryData, 0);
        }

        switch (argQueryType) {
            case AND: {
                // Do AND compound query
                for (int varIndex = 1;
                        varIndex < argQueries.size(); ++varIndex) {
                    QueryData varNext =
                            ((QueryData) argQueries.elementAt(varIndex));
                    varQueryData = new QueryData(
                            "", "(" + varQueryData.queryId +
                            "*" + varNext.queryId + ")",
                            QueryComposer.and(
                                varQueryData.query, varNext.query));
                }
            }
            break;

            case OR: {
                // Do OR compound query
                for (int varIndex = 1;
                        varIndex < argQueries.size(); ++varIndex) {
                    QueryData varNext = ((QueryData)
                            argQueries.elementAt(varIndex));
                    varQueryData = new QueryData(
                            "", "(" + varQueryData.queryId +
                            "|" + varNext.queryId + ")",
                            QueryComposer.or(
                                    varQueryData.query, varNext.query));
                }
            }
            break;

            case NOT: {
                // Do a NOT query
                if (varQueryData != null) {
                    varQueryData = new QueryData("",
                            "~(" + varQueryData.queryId + ")",
                            QueryComposer.not(varQueryData.query));
                }
            }
            break;

            default:
                // Do nothing
                break;
        }

        return varQueryData;
    }

    /**
     * Returns a vector of selected QueryData items
     *
     * @return Vector of QueryData items corresponding to each selected item
     *         in list
     */
    private Vector getSelectedItems() {
        boolean[] varFlags = new boolean[size()];
        Vector varQueries = new Vector(getSelectedFlags(varFlags));

        int varLength = varFlags.length;
        for (int varindex = 0; varindex < varLength; ++varindex) {
            QueryData varQuery =
                    (QueryData) equivalentQueries.elementAt(varindex);

            if (varFlags[varindex] && varQuery != null) {
                // Add selected query data to be executed later
                varQueries.addElement(varQuery);
            }
        }

        return varQueries;
    }

    /**
     * Deselects all selected items in the list
     */
    private void deselectAllItems() {
        setSelectedFlags(new boolean[size()]);
    }
}

// End of File

