/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.ui;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.StringItem;
import javax.microedition.broadcast.ServiceComponent;
import javax.microedition.broadcast.esg.ProgramEvent;
import javax.microedition.broadcast.esg.Service;
import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.broadcast.esg.Attribute;
import javax.microedition.broadcast.esg.NumericAttribute;
import javax.microedition.broadcast.esg.DateAttribute;
import javax.microedition.broadcast.esg.BooleanAttribute;
import javax.microedition.broadcast.esg.StringAttribute;
import javax.microedition.broadcast.esg.CommonMetadataSet;
import javax.microedition.broadcast.purchase.PurchaseObject;
import java.util.Date;

/**
 * Displays information from the specified service guide item
 */
public class ServiceGuideDataInfo extends Form implements CommandListener {
    // Next displayable

    private Displayable nextDisplayable;
    // Display of the application
    private Display display;
    // Service guide data object
    protected ServiceGuideData sgData;
    // Commands
    protected Command cmdBack;

    /**
     * Constructor
     */
    public ServiceGuideDataInfo(
            String argTitle,
            Displayable argNextDisplayable,
            ServiceGuideData argData) {
        super(argTitle);

        nextDisplayable = argNextDisplayable;
        sgData = argData;
        display = ViewHandler.getDisplay();

        cmdBack = new Command("Back", Command.BACK, 2);

        addCommand(cmdBack);

        setCommandListener(this);
        display.setCurrent(this);

        reloadMetadata();
    }

    /**
     * Handles command which is delegated to this object
     * @param argCommand Command to handle
     * @param argDisplayable View owning this view
     */
    public void commandAction(Command argCommand, Displayable argDisplayable) {
        if (argCommand == cmdBack) {
            display.setCurrent(nextDisplayable);
        }
    }

    /**
     * Clears previous and loads new service guide item's metadata
     * in the UI.
     */
    protected void reloadMetadata() {
        deleteAll();
        refreshMetadata();
    }

    /**
     * Refreshes the service guide item's meta data in the UI
     */
    protected void refreshMetadata() {
        addServiceMetaData();
        addProgramMetaData();
        addPurchaseMetaData();
        addServiceComponentMetaData();
    }

    private void addServiceMetaData() {
        if (!(sgData instanceof Service)) {
            return;
        }

        Attribute[] varAttributes =
                CommonMetadataSet.getInstance().getValidServiceAttributes();

        for (int varIndex = 0; varIndex < varAttributes.length; varIndex++) {
            addAttributeText(varAttributes[varIndex]);
        }
    }

    private void addProgramMetaData() {
        if (!(sgData instanceof ProgramEvent)) {
            return;
        }

        Attribute[] varAttributes =
                CommonMetadataSet.getInstance().getValidProgramAttributes();

        for (int varIndex = 0; varIndex < varAttributes.length; varIndex++) {
            addAttributeText(varAttributes[varIndex]);
        }
    }

    private void addPurchaseMetaData() {
        if (!(sgData instanceof PurchaseObject)) {
            return;
        }

        Attribute[] varAttributes =
                CommonMetadataSet.getInstance().getValidPurchaseAttributes();

        for (int varIndex = 0; varIndex < varAttributes.length; varIndex++) {
            addAttributeText(varAttributes[varIndex]);
        }
    }

    private void addServiceComponentMetaData() {
        if (!(sgData instanceof ServiceComponent)) {
            return;
        }

        Attribute[] varAttributes =
                CommonMetadataSet.getInstance().
                getValidServiceComponentAttributes();

        for (int varIndex = 0; varIndex < varAttributes.length; varIndex++) {
            addAttributeText(varAttributes[varIndex]);
        }
    }

    private void addAttributeText(Attribute argAttribute) {
        try {
            append("\n" + argAttribute.getName() + ":");
            if (argAttribute instanceof StringAttribute) {
                append("String:");
                String[] varValues =
                        sgData.getStringValues((StringAttribute) argAttribute);

                for (int varIndex = 0;
                        varIndex < varValues.length; varIndex++) {
                    addText(argAttribute.getName(), varValues[varIndex]);
                }
            } else if (argAttribute instanceof DateAttribute) {
                append("Date:");
                Date[] varValues =
                        sgData.getDateValues((DateAttribute) argAttribute);

                for (int varIndex = 0;
                        varIndex < varValues.length; varIndex++) {
                    addText(argAttribute.getName(),
                            varValues[varIndex].toString());
                }
            } else if (argAttribute instanceof BooleanAttribute) {
                append("Bool:");
                boolean[] varValues =
                        sgData.getBooleanValues(
                                (BooleanAttribute) argAttribute);

                for (int varIndex = 0;
                        varIndex < varValues.length; varIndex++) {
                    addText(argAttribute.getName(),
                            new Boolean(varValues[varIndex]).toString());
                }
            } else if (argAttribute instanceof NumericAttribute) {
                append("Numeric:");
                NumericAttribute varNumAttribute =
                        (NumericAttribute) argAttribute;
                if (varNumAttribute.getType() ==
                        NumericAttribute.FLOATING_POINT_TYPE) {
                    append("Float:");
                    double[] varValues =
                            sgData.getDoubleValues(varNumAttribute);
                    for (int varIndex = 0;
                            varIndex < varValues.length; varIndex++) {
                        addText(argAttribute.getName(),
                                Double.toString(varValues[varIndex]));
                    }
                } else {
                    append("Integer:");
                    long[] varValues = sgData.getLongValues(varNumAttribute);
                    for (int varIndex = 0;
                            varIndex < varValues.length; varIndex++) {
                        addText(argAttribute.getName(),
                                Long.toString(varValues[varIndex]));
                    }
                }
            } else {
                append("Unknown Type");
            }
        } catch (Exception e) {
            System.out.println(
                    "ServiceGuideDataInfo.refreshMetadata() - exception: "
                    + e.toString());
            append("Unknown Value\n");
        }
    }

    /**
     * Adds a new TextField to this form
     * @param aTitle Title of the text field
     * @param argValue Value of the text field
     */
    private void addText(String argTitle, String argValue) {
        StringItem varTextField = new StringItem(argTitle + ":", argValue);
        varTextField.setFont(ViewUtils.getDefaultFont());
        append(varTextField);
    }
}

// End of File

