/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
*/

// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.ServiceGuideController;
import com.nokia.example.engine.PresentationController;
import com.nokia.example.engine.SubscriptionController;

// EXTERNAL IMPORTS
import java.io.IOException;
import javax.microedition.io.Connector;
import javax.microedition.io.InputConnection;
import javax.microedition.io.Connection;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.broadcast.esg.Service;
import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.broadcast.esg.ServiceGuideListener;
import javax.microedition.broadcast.esg.CommonMetadataSet;
import javax.microedition.broadcast.esg.DataUnavailableException;
import javax.microedition.broadcast.connection.BroadcastConnection;
import javax.microedition.broadcast.connection.BroadcastFileConnection;
import javax.microedition.broadcast.connection.BroadcastConnectionListener;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

public class ServicesView extends List
        implements ViewBase, ServiceGuideListener, BroadcastConnectionListener
{
    // Service guide controller
    private ServiceGuideController serviceGuideController;

    // Presentation controller
    private PresentationController presentationController;

    // Subscription controller
    private SubscriptionController subscriptionController;

    // List of available services
    private Vector services;
    // Hashtable of connections
    private Hashtable connections;

    // Commands
    private Command cmdBack;
    private Command cmdListPrograms;
    private Command cmdSelect;
    private Command cmdShowInfo;
    private Command cmdListPurchaseObjects;
    private Command cmdListRelatedItems;

    // Initializer
    private Initializer initializer;

    /**
     * Constructor
     */
    ServicesView()
    {
        super("Services", List.IMPLICIT);

        cmdBack = new Command("Back", Command.BACK, 1);
        cmdListPrograms = new Command("Get programs", Command.SCREEN, 1);
        cmdSelect = new Command("Select", Command.SCREEN, 1);
        cmdShowInfo = new Command("Show details", Command.SCREEN, 1);
        cmdListPurchaseObjects =
                new Command("Get purchase objects", Command.SCREEN, 1);
        cmdListRelatedItems =
                new Command("Show related files", Command.SCREEN, 1);

        addCommand(cmdBack);
        addCommand(cmdListPrograms);
        addCommand(cmdShowInfo);
        addCommand(cmdListPurchaseObjects);
        addCommand(cmdListRelatedItems);
        setSelectCommand(cmdSelect);

        serviceGuideController = ServiceGuideController.getInstance();
        presentationController = PresentationController.getInstance();

        services = new Vector();
        connections = new Hashtable();

        // Initializer for full update
        initializer = new Initializer();
        initializer.start();
    }

    /**
     * From ServiceGuideListener
     */
    public void serviceGuideUpdated(
        String argEvent,
        ServiceGuideData argServiceGuideData,
        Object argEventData)
    {
        if (argEvent.equalsIgnoreCase(ServiceGuideListener.NEW_SERVICE_LISTED))
        {
            synchronized (this)
            {
                if (!services.contains(argServiceGuideData))
                {
                    addService((Service)argServiceGuideData);
                }
            }
        }
        else if (
                argEvent.equalsIgnoreCase(ServiceGuideListener.SERVICE_DELETED))
        {
            synchronized (this)
            {
                removeService((Service)argServiceGuideData);
            }
        }
        else if (
                argEvent.equalsIgnoreCase(
                        ServiceGuideListener.SERVICE_GUIDE_BULK_CHANGED))
        {
            requestUpdate();
        }
        else if (
                argEvent.equalsIgnoreCase(
                        ServiceGuideListener.SERVICE_GUIDE_DELETED))
        {
            synchronized (this)
            {
                services.removeAllElements();
                deleteAll();
                connections.clear();
            }
        }
    }

    /**
     * From BroadcastFileConnectionListener
     */
    public void update(
        BroadcastConnection argConnection,
        String argEvent,
        Object argEventData)
    {
        Service varService = (Service)connections.get(argConnection);
        try
        {
            if (varService != null &&
                    argEvent == BroadcastConnectionListener.AVAILABLE)
            {
                int varServiceIndex = services.indexOf(varService);
                set(varServiceIndex,
                    getString(varServiceIndex),
                    readLogo(varService, argConnection));

                argConnection.removeListener(this);
                // Remove from connections table
                connections.remove(argConnection);
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
            // Ignore
        }
    }

    /**
     * Activates the view
     */
    public void activate()
    {
        subscriptionController = SubscriptionController.getInstance();

        requestUpdate();
        serviceGuideController.addListener(this);
    }

    /**
     * Deactivates the view
     */
    public void deactivate()
    {
        // Do not remove listener as the service guide might get invalidated
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable()
    {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param aListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener argListener)
    {
        super.setCommandListener(argListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param argCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable)
    {
        if (argCommand == cmdBack)
        {
            ViewHandler.activateView(ViewHandler.PLATFORM_PROVIDER_VIEW_ID);
            return;
        }

        int index = getSelectedIndex();
        if (index < 0)
        {
            // Nothing selected.
            return;
        }

        Service varService = (Service)services.elementAt(index);

        if (argCommand == cmdListPrograms)
        {
            serviceGuideController.setService(varService);
            ViewHandler.activateView(ViewHandler.PROGRAMS_VIEW_ID);
        }
        else if (argCommand == cmdSelect)
        {
            presentationController.prepare(varService);
            ViewHandler.activateView(ViewHandler.VIDEO_CANVAS_VIEW_ID);
        }
        else if (argCommand == cmdShowInfo)
        {
            new ServiceGuideDataInfo(getString(index), this, varService);
        }
        else if (argCommand == cmdListPurchaseObjects)
        {
            String varServiceId;
            try
            {
                varServiceId = (String)varService.getStringValue(
                                CommonMetadataSet.SERVICE_ID);
            }
            catch (DataUnavailableException due)
            {
                due.printStackTrace();
                return;
            }

            subscriptionController.setId(varServiceId);
            ViewHandler.activateView(ViewHandler.PURCHASE_OBJECTS_VIEW_ID);
        }
        else if (argCommand == cmdListRelatedItems)
        {
            String varRelatedMaterial;
            try
            {
                varRelatedMaterial = (String)varService.getStringValue(
                                      CommonMetadataSet.SERVICE_REL_MATERIAL);
            }
            catch (DataUnavailableException due)
            {
                due.printStackTrace();
                return;
            }
            if (varRelatedMaterial != null)
            {
                ViewHandler.getDisplay().setCurrent(
                        new ShowFilesView(varRelatedMaterial));
            }
        }
    }

    /**
     * Adds a new service
     *
     * @param aService The service to be added
     */
    private synchronized void addService(Service argService)
    {
        services.addElement(argService);

        String varName = null;
        try
        {
            // List service names
            varName = argService.getStringValue(CommonMetadataSet.SERVICE_NAME);
        }
        catch (DataUnavailableException due)
        {
            varName = "(Unnamed)";
        }

        // Add default icon for now
        append(varName, null);
        int varIndex = services.size() - 1;
        setFont(varIndex, ViewUtils.getDefaultFont());
    }

    private synchronized void removeService(Service argService)
    {
        int varIndex = services.indexOf(argService);
        if (varIndex != -1)
        {
            services.removeElement(argService);
            delete(varIndex);

            // remove connection if available
            Enumeration varElements = connections.keys();
            while (varElements.hasMoreElements())
            {
                BroadcastFileConnection varCon =
                    (BroadcastFileConnection)varElements.nextElement();
                if (connections.get(varCon) == argService)
                {
                    connections.remove(varCon);
                    varCon.removeListener(this);
                    try
                    {
                        varCon.close();
                    }
                    catch (Exception e)
                    {
                        // Ignore
                    }
                }
            }
        }
    }

    private synchronized void requestUpdate()
    {
        // Notify updater if not running
        try
        {
            synchronized (initializer)
            {
                initializer.notify();
            }
        }
        catch (Exception e)
        {
        }
    }

    private synchronized void updateLogos()
    {
        int varCount = services.size();
        for (int varIndex = 0; varIndex < varCount; varIndex++)
        {
            Service varService = (Service)services.elementAt(varIndex);
            // Update only if no logo available
            if (getImage(varIndex) == null && !connections.contains(varService))
            {
                try
                {
                    String varLogo = 
                            varService.getStringValue(
                                    CommonMetadataSet.SERVICE_AUX_LOGO);
                    Connection varLogoConnection =
                            Connector.open(varLogo, Connector.READ);
                    Image varImage = readLogo(varService, varLogoConnection);

                    // Set image if it was available
                    if (varImage != null)
                    {
                        String varString = getString(varIndex);
                        set(varIndex, varString, varImage);
                    }
                }
                catch (Exception e)
                {
                    // No image
                    e.printStackTrace();
                }
            }
        }
    }

    private Image readLogo(Service aService, Connection aConnection)
    {
        Image varImage = null;

        try
        {
            // Check if this is a broadcast file connection
            if (aConnection instanceof BroadcastFileConnection)
            {
                BroadcastFileConnection varConnection =
                        (BroadcastFileConnection)aConnection;

                System.out.println(
                        "ServicesView.readLogo(), logo URL=" +
                        varConnection.getURL());

                if (!varConnection.isAvailable())
                {
                    connections.put(varConnection, aService);
                    varConnection.addListener(this);
                }
                else
                {
                    varImage =
                            Image.createImage(varConnection.openInputStream());
                    varConnection.close();
                }
            }
            // Standard input connection
            else if (aConnection instanceof InputConnection)
            {
                InputConnection varConnection = (InputConnection)aConnection;
                varImage = Image.createImage(varConnection.openInputStream());
                varConnection.close();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        return varImage;
    }

    /**
     * Updates the services' list
     */
    private synchronized void updateServices()
    {
        // Executes either query or returns all available services
        Vector varServices = serviceGuideController.executeQuery();

        // Check withc services have been removed
        Enumeration varCurrentServices = services.elements();
        while (varCurrentServices.hasMoreElements())
        {
            Service varService = (Service)varCurrentServices.nextElement();
            if (!varServices.contains(varService))
            {
                removeService(varService);
            }
        }

        // Check that which services are new
        Enumeration varNewServices = varServices.elements();
        while (varNewServices.hasMoreElements())
        {
            Service varService = (Service)varNewServices.nextElement();
            if (!services.contains(varService))
            {
                addService(varService);
            }
        }
    }

    /**
     * Inner class which is used for service list initialization
     */
    private class Initializer extends Thread
    {
        /**
         * Constructor
         */
        public Initializer()
        {
        }

        /**
         * Sets the update mode
         */
        public void setUpdateMode()
        {
        }

        /**
         * Run method for this thread
         */
        public void run()
        {
            while (true)
            {
                // Processing done. Start wainting
                try
                {
                    synchronized (this)
                    {
                        this.wait();
                    }

                    // Full update
                    updateServices();
                    updateLogos();
                }
                catch (Exception e)
                {
                    // Interrupted or service guide invalidated
                    e.printStackTrace();
                }
            }
        }
    }
}

// End of File
