/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
*/

package com.nokia.example.ui;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;

import javax.microedition.broadcast.connection.BroadcastConnection;
import javax.microedition.broadcast.connection.BroadcastConnectionListener;
import javax.microedition.broadcast.connection.BroadcastFileConnection;
import javax.microedition.io.Connector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.List;

public class ShowFilesView extends List implements Runnable,
        BroadcastConnectionListener, CommandListener
{
    private Command backCommand;
    private Command selectCommand;
    private String fileLocation;
    private BroadcastFileConnection broadFileConn;

    public ShowFilesView(String argFileLocation)
    {
        super(argFileLocation, List.EXCLUSIVE);
        selectCommand = new Command("Show file as image", Command.ITEM, 1);
        addCommand(selectCommand);
        fileLocation = argFileLocation;
        backCommand = new Command("Back", Command.BACK, 1);
        addCommand(backCommand);
        setCommandListener(this);
        new Thread(this).start();
    }

    public void run()
    {
        try
        {
            broadFileConn = (BroadcastFileConnection)
                    Connector.open(fileLocation, Connector.READ);
            broadFileConn.addListener(this);

            // List does not return anything until the file description table
            // has been received.
            boolean varWaitingFileList = true;
            while (varWaitingFileList)
            {
                Enumeration varEnum = broadFileConn.list();
                while (varEnum.hasMoreElements())
                {
                    BroadcastFileConnection varFile =
                            (BroadcastFileConnection) varEnum.nextElement();
                    varFile.addListener(this);
                    addItem(varFile.getName());
                    varWaitingFileList = false;
                }
            }
        }
        catch (Throwable e)
        {
            addItem(e.toString());
        }
    }

    private void addItem(String argMessage)
    {
        append(argMessage, null);
    }

    public void update(BroadcastConnection argConnection,
            String argEvent, Object argData)
    {
        //BroadcastFileConnection bfc = (BroadcastFileConnection)aConnection;
        System.out.println("BroadcastConnection update ++++ ");
        BroadcastFileConnection varFileCon =
                (BroadcastFileConnection)argConnection;
        try
        {
            if (!varFileCon.isDirectory())
            {
                int varIndex = findIndex(varFileCon);
                set(varIndex , "A: " + getString(varIndex), getImage(varIndex));
            }
        }
        catch (IOException e)
        {
            addItem("IOE: " + e.getMessage());
        }
    }

    private BroadcastFileConnection findConnection(String argName)
            throws IOException
    {
        Enumeration varEnum = broadFileConn.list();
        while (varEnum.hasMoreElements())
        {
            BroadcastFileConnection varFileCon =
                    (BroadcastFileConnection) varEnum.nextElement();
            if (varFileCon.getName().equals(argName))
            {
                return varFileCon;
            }
        }
        return null;
    }

    private int findIndex(BroadcastFileConnection argConnection)
    {
        String varName = argConnection.getName();
        int varCount = size();
        for (int varIndex = 0; varIndex < varCount; varIndex++)
        {
            if (getString(varIndex).equals(varName))
            {
                return varIndex;
            }
        }
        return -1;
    }

    private void makeConnection(int argIndex) throws IOException
    {
        String varName = getString(argIndex);
        if (varName.startsWith("A: "))
        {
            varName = varName.substring(3);
        }
        BroadcastFileConnection varFile = findConnection(varName);
        InputStream varInStream = varFile.openInputStream();
        ByteArrayOutputStream varBaos = new ByteArrayOutputStream();
        int varValue = 0;
        while ((varValue = varInStream.read()) != -1)
        {
            varBaos.write(varValue);
        }
        varBaos.flush();
        byte[] varImageBytes = varBaos.toByteArray();
        Image varImage =
                Image.createImage(varImageBytes, 0, varImageBytes.length);
        set(argIndex, getString(argIndex), varImage);
    }

    public void commandAction(Command argCommand, Displayable argDisplayable)
    {
        if (argCommand == backCommand)
        {
            ViewHandler.activateView(ViewHandler.SERVICES_VIEW_ID);
        }
        if (argCommand == selectCommand)
        {
            try
            {
                makeConnection(getSelectedIndex());
            }
            catch (IOException e)
            {
                addItem("IO:" + e.getMessage());
            }
            catch (NullPointerException npe)
            {
                addItem("Could not find connection");
            }
        }

    }

}
