/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.ServiceGuideController;

// EXTERNAL IMPORTS
import java.util.Vector;
import java.util.Enumeration;
import java.io.PrintStream;
import java.io.OutputStream;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.Gauge;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.AlertType;
import javax.microedition.broadcast.esg.Attribute;
import javax.microedition.broadcast.esg.DateAttribute;
import javax.microedition.broadcast.esg.StringAttribute;
import javax.microedition.broadcast.esg.NumericAttribute;
import javax.microedition.broadcast.esg.BooleanAttribute;
import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.broadcast.esg.DataUnavailableException;

public class SortAttributesView extends List
        implements ViewBase, CommandListener {
    // MEMBERS

    private Vector selectedAttributes;
    private AddSortAttributes addSortAttributes;
    private ServiceGuideController serviceGuideController;
    // Commands
    private Command cmdDumpPrograms;
    private Command cmdDumpServices;
    private Command cmdReset;
    private Command cmdAttributeAdd;
    private Command cmdAdd;
    private Command cmdAttributeBack;
    private Command cmdBack;

    /**
     * Constructor
     */
    SortAttributesView() {
        super("Sort Attributes", List.EXCLUSIVE);

        cmdDumpPrograms = new Command("Dump programs", Command.SCREEN, 7);
        cmdDumpServices = new Command("Dump services", Command.SCREEN, 6);
        cmdReset = new Command("Reset attributes", Command.SCREEN, 5);
        cmdAttributeAdd = new Command("Add", Command.SCREEN, 3);
        cmdAdd = new Command("Add attributes", Command.SCREEN, 3);
        cmdAttributeBack = new Command("Back", Command.BACK, 2);
        cmdBack = new Command("Back", Command.BACK, 2);

        selectedAttributes = new Vector();
        addSortAttributes = new AddSortAttributes(
                "Add Attribute",
                this,
                selectedAttributes);
        addSortAttributes.addCommand(cmdAttributeBack);
        addSortAttributes.addCommand(cmdAttributeAdd);
        addSortAttributes.setCommandListener(this);

        serviceGuideController = ServiceGuideController.getInstance();
    }

    /**
     * Activates the view
     */
    public void activate() {
        addCommand(cmdBack);
        addCommand(cmdAdd);
        addCommand(cmdReset);
        addCommand(cmdDumpServices);
        addCommand(cmdDumpPrograms);
    }

    /**
     * Deactivates the view
     */
    public void deactivate() {
        removeCommand(cmdDumpPrograms);
        removeCommand(cmdDumpServices);
        removeCommand(cmdReset);
        removeCommand(cmdAdd);
        removeCommand(cmdBack);
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable() {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param argListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener argListener) {
        super.setCommandListener(argListener);
    }

    /**
     * Handles a command which is delegated to this object via CommandListener
     * interface
     */
    public void commandAction(Command argCommand, Displayable argDisplayable) {
        if (argCommand == cmdAttributeBack) {
            addSortAttributes.deactivate();
            refreshList();
        } else if (argCommand == cmdAttributeAdd) {
            addSortAttributes.add();
        }
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param argCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable) {
        if (argCommand == cmdBack) {
            ViewHandler.activateView(ViewHandler.PLATFORM_PROVIDER_VIEW_ID);
        } else if (argCommand == cmdAdd) {
            addSortAttributes.activate();
        } else if (argCommand == cmdReset) {
            selectedAttributes.removeAllElements();
            addSortAttributes.reset();
            refreshList();
        } else if (argCommand == cmdDumpServices) {
            (new DumpQueryResults(
                    ServiceGuideController.QUERY_SERVICES)).start();
        } else if (argCommand == cmdDumpPrograms) {
            (new DumpQueryResults(
                    ServiceGuideController.QUERY_PROGRAMS)).start();
        }
    }

    /**
     * Refreshes the selected attribute item list on the display
     */
    private void refreshList() {
        deleteAll();
        int varSize = selectedAttributes.size();
        for (int varIndex = 0; varIndex < varSize; ++varIndex) {
            Attribute varAttribute =
                    (Attribute) selectedAttributes.elementAt(varIndex);
            append(varAttribute.getName(), null);
            setFont(varIndex, ViewUtils.getDefaultFont());
        }
        serviceGuideController.setSortKeys(selectedAttributes);
    }

    /**
     * Retrieves attributes value from given ServiceGuideData object
     */
    private Object getValue(
            ServiceGuideData argServiceGuideData,
            Attribute argAttribute) {
        Object varValue = null;
        try {
            if (argAttribute instanceof DateAttribute) {
                varValue = argServiceGuideData.getDateValue(
                        (DateAttribute) argAttribute);
            } else if (argAttribute instanceof StringAttribute) {
                varValue = argServiceGuideData.getStringValue(
                        (StringAttribute) argAttribute);
            } else if (argAttribute instanceof NumericAttribute) {
                switch (((NumericAttribute) argAttribute).getType()) {
                    case NumericAttribute.FLOATING_POINT_TYPE: {
                        double varResult =
                                argServiceGuideData.getDoubleValue(
                                (NumericAttribute) argAttribute);
                        varValue = new Double(varResult);
                    }
                    break;

                    default: {
                        long varResult =
                                argServiceGuideData.getLongValue(
                                (NumericAttribute) argAttribute);
                        varValue = new Long(varResult);
                    }
                    break;
                }
            } else if (argAttribute instanceof BooleanAttribute) {
                boolean varResult =
                        argServiceGuideData.getBooleanValue(
                        (BooleanAttribute) argAttribute);
                varValue = new Boolean(varResult);
            }
            // else
            // was an object attribute, dont return value for now

        } catch (DataUnavailableException due) {
            // Ok, value was just not available
        }

        return varValue;
    }

    /**
     * Inner helper class for displaying progress note while dumping service
     * guide data
     */
    private class DumpQueryResults extends Thread {
        // CONSTANTS

        private final static int MAX_PROGRESS = 100;
        // MEMBERS
        private int queryType;
        // Progress of the service guide data dumping
        private Gauge progress;
        private AlertNote progressNote;

        /**
         * Constructor
         */
        DumpQueryResults(int argQueryType) {
            queryType = argQueryType;
            progress = new Gauge(null, false, MAX_PROGRESS, 0);
            progressNote = new AlertNote("Dumping data",
                    "Writing service guide data...",
                    AlertType.INFO,
                    getDisplayable());
            progressNote.setIndicator(progress);
            progressNote.addCommand(new Command("Hide", Command.STOP, 3));
        }

        /**
         * Thread access point
         */
        public void run() {
            progressNote.execute();
            serviceGuideController.setQuery(null, queryType);
            printQueryResult(
                    System.out, serviceGuideController.executeQuery());
        }

        /**
         * Prints query result into given output stream
         */
        private void printQueryResult(
                OutputStream argOutput, Vector argQueryResult) {
            PrintStream varStream = new PrintStream(argOutput);
            varStream.println("<--- START QUERY RESULT DUMP --->");

            int varResults = argQueryResult.size();
            double varCoeff = MAX_PROGRESS / (double) varResults;
            for (int varIndex = 0; varIndex < varResults; ++varIndex) {
                progress.setValue((int) ((varIndex + 1) * varCoeff));

                varStream.println("ITEM #" + varIndex + ":");
                ServiceGuideData varSGData =
                        (ServiceGuideData) argQueryResult.elementAt(varIndex);

                int varAttribs = selectedAttributes.size();
                for (int varIndex2 = 0; varIndex2 < varAttribs; ++varIndex2) {
                    Attribute varAttrib =
                            (Attribute) selectedAttributes.elementAt(varIndex2);

                    String varStr = 
                            " ATTRIB #" + varIndex2 + ": " +
                            varAttrib.getName() + " = ";
                    Object varValue = getValue(varSGData, varAttrib);
                    if (varValue != null) {
                        varStr += varValue.toString();
                    } else {
                        varStr += "null";
                    }
                    varStream.println(varStr);
                }
                varStream.println();
            }

            varStream.println("<--- END QUERY RESULT DUMP --->");
            varStream.flush();
            varStream.close();

            // Dismiss progress note
            ViewHandler.getDisplay().setCurrent(getDisplayable());
        }
    }
}

// End of File

