/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.ServiceGuideController;
import com.nokia.example.engine.SubscriptionController;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.StringItem;
import javax.microedition.broadcast.esg.DataUnavailableException;

/**
 * Shows information related to subscriptions.
 */
public class SubscriptionView extends Form implements ViewBase {
    // Controller handling subscriptions.

    SubscriptionController subscriptionController;
    // Commands
    private Command cmdListPurchaseObjects;
    private Command cmdListSubscriptions;
    private Command cmdBack;

    /**
     * Constructor
     */
    SubscriptionView() {
        super("Subscriptions");

        cmdListPurchaseObjects =
                new Command("List purchase objects", Command.SCREEN, 1);
        cmdListSubscriptions =
                new Command("List subscriptions", Command.SCREEN, 1);
        cmdBack = new Command("Back", Command.BACK, 1);

        addCommand(cmdListPurchaseObjects);
        addCommand(cmdListSubscriptions);
        addCommand(cmdBack);
    }

    /**
     * Activates the view
     */
    public void activate() {
        deleteAll();

        subscriptionController = SubscriptionController.getInstance();

        String varDefaultChannel = subscriptionController.getDefaultChannel();
        varDefaultChannel =
                (varDefaultChannel == null ? "[null]" : varDefaultChannel);
        StringItem varStringItem =
                new StringItem("Default channel", varDefaultChannel);
        varStringItem.setFont(ViewUtils.getDefaultFont());
        append(varStringItem);

        String[] varSupportedChannels =
                subscriptionController.getSupportedChannels();

        for (int varIndex = 0;
                varIndex < varSupportedChannels.length; varIndex++) {
            varStringItem = 
                    new StringItem("Channel(" + varIndex + ")",
                            varSupportedChannels[varIndex]);
            varStringItem.setFont(ViewUtils.getDefaultFont());
            append(varStringItem);
        }
    }

    /**
     * Deactivates the view
     */
    public void deactivate() {
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable() {
        return this;
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param aCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable) {
        if (argCommand == cmdBack) {
            ViewHandler.activatePreviousView();
        } else if (argCommand == cmdListPurchaseObjects) {
            // List all purchase objects (id=null).
            subscriptionController.setId(null);
            ViewHandler.activateView(ViewHandler.PURCHASE_OBJECTS_VIEW_ID);
        } else if (argCommand == cmdListSubscriptions) {
            ViewHandler.activateView(ViewHandler.SUBSCRIBED_OBJECTS_VIEW_ID);
        }
    }
}

// End of File

