/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.ServiceGuideController;
import com.nokia.example.engine.PresentationController;
import com.nokia.example.engine.PresentationListener;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;
import javax.microedition.media.control.VideoControl;
import javax.microedition.media.control.VolumeControl;
import javax.microedition.broadcast.ServiceComponent;
import javax.microedition.broadcast.esg.Service;
import javax.microedition.broadcast.esg.CommonMetadataSet;
import com.nokia.mid.media.AudioOutputControl;
import java.util.Vector;

/**
 * Displays the current broadcast stream in a canvas
 */
public class VideoCanvasView extends Canvas
        implements ViewBase, PresentationListener {
    // Service guide controller

    private ServiceGuideController serviceGuide;
    private PresentationController controller;
    // Commands
    private Command cmdBack;
    private Command cmdStartContext;
    private Command cmdStopContext;
    private Command cmdStartPlayer;
    private Command cmdStopPlayer;
    private Command cmdClosePlayer;
    private Command cmdDeallocatePlayer;
    private Command cmdSetVisible;
    private Command cmdHelp;
    private Command cmdCanvasFullScreen;
    private Command cmdShowServiceComponents;
    private Command cmdExit;
    private Command cmdToggleSpeaker;
    // Thread for updating title pane
    private Thread thread;
    private Vector services;
    private Service currentService;
    // Fullscreen flag
    private boolean fullscreen = false;
    private boolean canvasFullScreen = false;
    // Video step size
    private static final int STEP = 10;
    // Thread sleep time
    private static final int THREAD_SLEEP_TIME = 2500;
    // Help text
    private static final String HELP_TEXT =
            "Keys can be used to manipulate the video\n"
            + "\n"
            + "1 increases the width\n"
            + "7 decreases the width\n"
            + "3 increases the height\n"
            + "9 decreases the height\n"
            + "5 and up key toggle fullscreen mode of the video\n"
            + "\n"
            + "Keys 2, 4, 8, 6 can be used to move the video\n"
            + "\n"
            + "Left and right keys change the presented service. "
            + "Down key sets video visible\n"
            + "\n"
            + "* decreases the current volume level\n"
            + "# increases the current volume level\n"
            + "0 mutes the volume\n";

    /**
     * Constructor
     */
    VideoCanvasView() {
        super();

        cmdBack = new Command("Back", Command.BACK, 1);

        cmdStartContext = new Command("Start context", Command.SCREEN, 1);
        cmdStopContext = new Command("Stop context", Command.SCREEN, 1);

        cmdStartPlayer = new Command("Start player", Command.SCREEN, 1);
        cmdStopPlayer = new Command("Stop player", Command.SCREEN, 1);
        cmdClosePlayer = new Command("Close player", Command.SCREEN, 1);
        cmdDeallocatePlayer =
                new Command("Deallocate player", Command.SCREEN, 1);

        cmdSetVisible = new Command("Set visible", Command.SCREEN, 2);
        cmdHelp = new Command("Help", Command.SCREEN, 3);
        cmdCanvasFullScreen =
                new Command("Toggle full canvas", Command.SCREEN, 2);
        cmdToggleSpeaker =
                new Command("Toggle Loud Speaker", Command.SCREEN, 2);

        cmdShowServiceComponents =
                new Command("List components", Command.SCREEN, 3);

        cmdExit = new Command("Exit", Command.EXIT, 10);


        addCommand(cmdBack);
        addCommand(cmdStartContext);
        addCommand(cmdStopContext);

        addCommand(cmdStartPlayer);
        addCommand(cmdStopPlayer);
        addCommand(cmdClosePlayer);
        addCommand(cmdDeallocatePlayer);

        addCommand(cmdSetVisible);
        addCommand(cmdHelp);
        addCommand(cmdCanvasFullScreen);
        addCommand(cmdToggleSpeaker);

        addCommand(cmdShowServiceComponents);

        addCommand(cmdExit);

        controller = PresentationController.getInstance();
        serviceGuide = ServiceGuideController.getInstance();
    }

    /**
     * From PresentationListener
     */
    public void notifyPresentationEvent(
            int argEventType,
            String argMessage) {
        if (argEventType == PresentationListener.EVENT_PRESENTING) {
            Player[] varPlayers = controller.getCurrentPlayers();

            // The players should all be realized at this point.
            // Initialize and display them.
            for (int varIndex = 0; varIndex < varPlayers.length; varIndex++) {
                Player varPlayer = varPlayers[varIndex];
                VideoControl varControl =
                        (VideoControl) varPlayer.getControl("VideoControl");

                if (varControl != null) {
                    try {
                        varControl.initDisplayMode(
                                VideoControl.USE_DIRECT_VIDEO, this);
                        // Show the video in the Canvas
                        varControl.setVisible(true);
                        varControl.setDisplayFullScreen(true);
                    } catch (MediaException ex) {
                        ex.printStackTrace();
                    }
                }

                VolumeControl varVolControl =
                        (VolumeControl) varPlayer.getControl("VolumeControl");
                if (varVolControl != null) {
                    varVolControl.setLevel(80);
                }
            }
        } else if (argEventType == PresentationListener.EVENT_ERROR) {
            AlertNote varAlert = new AlertNote("Error",
                    argMessage,
                    AlertType.ERROR,
                    this);
            varAlert.execute();
        }
    }

    /**
     * From Canvas
     */
    public void paint(Graphics argGraphics) {
        // Nothing to do here
    }

    /**
     * Activates the view
     */
    public void activate() {
        controller.addListener(this);
        controller.start();
        services = serviceGuide.executeQuery();
        currentService = controller.getCurrentService();

        thread = new Thread() {

            public void run() {
                while (thread != null) {
                    try {
                        synchronized (this) {
                            updateTitle();
                            sleep(THREAD_SLEEP_TIME);
                        }
                    } catch (Exception e) {
                        // Ignore
                    }
                }
            }
        };
        thread.start();
    }

    /**
     * Deactivates the view
     */
    public void deactivate() {
        controller.removeListener(this);
        thread = null;
        controller.stop();
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable() {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param argListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener argListener) {
        super.setCommandListener(argListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param argCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable) {
        try {
            doHandleCommand(argCommand, argDisplayable);
        } catch (Exception e) {
            e.printStackTrace();

            AlertNote varAlert =
                    new AlertNote(
                    "Exception", e.toString(), AlertType.ERROR, this);

            varAlert.execute();
        }
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param aCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    private void doHandleCommand(Command argCommand, Displayable argDisplayable)
            throws Exception {
        if (argCommand == cmdBack) {
            ViewHandler.activateView(ViewHandler.SERVICES_VIEW_ID);
        } else if (argCommand == cmdStopContext) {
            controller.stop();
        } else if (argCommand == cmdStartContext) {
            controller.start();
        } else if (argCommand == cmdStartPlayer) {
            Player[] varPlayers = controller.getCurrentPlayers();
            for (int varIndex = 0; varIndex < varPlayers.length; varIndex++) {
                varPlayers[varIndex].start();
            }
        } else if (argCommand == cmdStopPlayer) {
            Player[] varPlayers = controller.getCurrentPlayers();
            for (int varIndex = 0; varIndex < varPlayers.length; varIndex++) {
                varPlayers[varIndex].stop();
            }
        } else if (argCommand == cmdClosePlayer) {
            Player[] varPlayers = controller.getCurrentPlayers();
            for (int varIndex = 0; varIndex < varPlayers.length; varIndex++) {
                varPlayers[varIndex].close();
            }
        } else if (argCommand == cmdDeallocatePlayer) {
            Player[] varPlayers = controller.getCurrentPlayers();
            for (int varIndex = 0; varIndex < varPlayers.length; varIndex++) {
                varPlayers[varIndex].deallocate();
            }
        } else if (argCommand == cmdSetVisible) {
            VideoControl varControl = getVideoControl();
            if (varControl != null) {
                varControl.setVisible(true);
            }
        } else if (argCommand == cmdHelp) {
            new HelpContainer("Help", this, HELP_TEXT);
        } else if (argCommand == cmdCanvasFullScreen) {
            toggleCanvasFullscreen();
        } else if (argCommand == cmdToggleSpeaker) {
            Player[] varPlayers = controller.getCurrentPlayers();
            for (int varIndex = 0; varIndex < varPlayers.length; varIndex++) {
                toggleAudioOutput(varPlayers[varIndex]);
            }
        } else if (argCommand == cmdShowServiceComponents) {
            ViewHandler.activateView(ViewHandler.SERVICE_COMPONENTS_VIEW_ID);
        }
    }

    private void toggleAudioOutput(Player argPlayer) {
        AudioOutputControl varAOcontrol = 
                (AudioOutputControl) argPlayer.getControl(
                        "com.nokia.mid.media.AudioOutputControl");
        if (varAOcontrol != null) {
            try {
                System.out.println("Got audio output controller.");
                int varNowMode = varAOcontrol.getOutputMode();
                System.out.println(
                        "Got Mode audio output controller." + varNowMode);
                if (varNowMode != AudioOutputControl.PUBLIC) {
                    System.out.println("Setting audio output mode to PUBLIC");
                    varAOcontrol.setOutputMode(AudioOutputControl.PUBLIC);
                } else if (varNowMode == AudioOutputControl.PUBLIC) {
                    varAOcontrol.setOutputMode(AudioOutputControl.PRIVATE);
                    System.out.println("Setting audio output mode to Private");
                }
            } catch (Exception ie) {
                ie.printStackTrace();
            }
        } else {
            System.out.println("No audio output controller.");
        }
    }

    /**
     * Handles key events
     */
    public void keyPressed(int argKeyCode) {
        System.out.println("VideoCanvasView + keyCode = " + argKeyCode);

        // First we handle key event. If key event is a game event, it is 
        // handled separately in handleGameAction() method. Errors are ignored
        if (handleKeyEvent(argKeyCode)) {
            handleGameAction(argKeyCode);
        }
    }

    /**
     * Handles game actions
     */
    private void handleGameAction(int argKeyCode) {
        try {
            switch (getGameAction(argKeyCode)) {
                case LEFT: {
                    int varIndex = services.indexOf(currentService);
                    switchChannel(varIndex > 0 ?
                                varIndex - 1 : services.size() - 1);
                    break;
                }
                case RIGHT: {
                    int varIndex = services.indexOf(currentService);
                    switchChannel(
                            varIndex >= 0 && varIndex < services.size() - 1 ?
                                    varIndex + 1 : 0);
                    break;
                }
                case UP: {
                    toggleFullscreen();
                    break;
                }
                case DOWN: {
                    VideoControl varVideoController = getVideoControl();
                    varVideoController.setVisible(true);
                    break;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Handles key events. Not game actions
     */
    private boolean handleKeyEvent(int argKeyCode) {
        boolean varCheckGameAction = false;

        VideoControl varVidControl = getVideoControl();
        VolumeControl varVolControl = getVolumeControl();

        try {
            switch (argKeyCode) {
                // Video scaling operations
                case KEY_NUM1: {
                    varVidControl.setDisplaySize(
                            varVidControl.getDisplayWidth() + STEP,
                            varVidControl.getDisplayHeight());
                    break;
                }
                case KEY_NUM3: {
                    varVidControl.setDisplaySize(
                            varVidControl.getDisplayWidth(),
                            varVidControl.getDisplayHeight() + STEP);
                    break;
                }
                case KEY_NUM7: {
                    varVidControl.setDisplaySize(
                            varVidControl.getDisplayWidth() - STEP,
                            varVidControl.getDisplayHeight());
                    break;
                }
                case KEY_NUM9: {
                    varVidControl.setDisplaySize(
                            varVidControl.getDisplayWidth(),
                            varVidControl.getDisplayHeight() - STEP);
                    break;
                }
                case KEY_NUM5: {
                    toggleFullscreen();
                    break;
                }
                // Video moving operations
                case KEY_NUM2: {
                    varVidControl.setDisplayLocation(
                            varVidControl.getDisplayX(),
                            varVidControl.getDisplayY() - STEP);
                    break;
                }
                case KEY_NUM8: {
                    varVidControl.setDisplayLocation(
                            varVidControl.getDisplayX(),
                            varVidControl.getDisplayY() + STEP);
                    break;
                }
                case KEY_NUM4: {
                    varVidControl.setDisplayLocation(
                            varVidControl.getDisplayX() - STEP,
                            varVidControl.getDisplayY());
                    break;
                }
                case KEY_NUM6: {
                    varVidControl.setDisplayLocation(
                            varVidControl.getDisplayX() + STEP,
                            varVidControl.getDisplayY());
                    break;
                }
                // Volume handling
                case KEY_POUND: {
                    varVolControl.setLevel(varVolControl.getLevel() + STEP);
                    break;
                }
                case KEY_STAR: {
                    varVolControl.setLevel(varVolControl.getLevel() - STEP);
                    break;
                }
                case KEY_NUM0: {
                    varVolControl.setMute(!varVolControl.isMuted());
                    break;
                }
                // Service selections

                default: {
                    varCheckGameAction = true;
                    break;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            varCheckGameAction = true;
        }

        return varCheckGameAction;
    }

    /**
     * Get Player's VideoControl
     */
    private VideoControl getVideoControl() {
        VideoControl varVidControl = null;

        try {
            Player[] varPlayers = controller.getCurrentPlayers();
            for (int varIndex = 0; varIndex < varPlayers.length; varIndex++) {
                Player varPlayer = varPlayers[varIndex];
                varVidControl =
                        (VideoControl) varPlayer.getControl("VideoControl");
                if (varVidControl != null) {
                    break;
                }
            }
        } catch (Exception e) {
            // Incorrect state or no control
        }

        return varVidControl;
    }

    /**
     * Get Player's VolumeControl
     */
    private VolumeControl getVolumeControl() {
        VolumeControl varControl = null;

        try {
            Player[] varPlayers = controller.getCurrentPlayers();
            for (int varIndex = 0; varIndex < varPlayers.length; varIndex++) {
                Player varPlayer = varPlayers[varIndex];
                varControl =
                        (VolumeControl) varPlayer.getControl("VolumeControl");
                if (varControl != null) {
                    break;
                }
            }
        } catch (Exception e) {
            // Incorrect state or no control
        }

        return varControl;
    }

    /**
     * Toggles VideoControl's fullscreen setting
     */
    private void toggleFullscreen() {
        VideoControl varControl = getVideoControl();
        if (varControl != null) {
            try {
                fullscreen = !fullscreen;
                varControl.setDisplayFullScreen(fullscreen);
            } catch (Exception e) {
                // Restore previous setting
                fullscreen = !fullscreen;
            }
        }
    }

    /**
     * Updates the title of this view
     */
    private synchronized void updateTitle() {
        Service varService = controller.getCurrentService();
        String varTitle;

        try {
            varTitle =
                    varService.getStringValue(CommonMetadataSet.SERVICE_NAME);
        } catch (Exception e) {
            varTitle = "No title";
        }

        try {
            varTitle += " (Q:" +
                        Integer.toString(controller.getSignalQuality()) + "%)";
        } catch (Exception e) {
            // Ignored intentionally
        }

        setTitle(varTitle);
    }

    /**
     * Switches to the given channel
     */
    private void switchChannel(int argChannelIndex) {
        Service varService = (Service) services.elementAt(argChannelIndex);
        controller.prepare(varService);
        controller.start();
        currentService = controller.getCurrentService();
        updateTitle();
    }

    /**
     * Toggles Canvas' fullscreen setting
     */
    private void toggleCanvasFullscreen() {
        canvasFullScreen = !canvasFullScreen;
        setFullScreenMode(canvasFullScreen);
    }
}

// End of File

