/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
*/

// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.PresentationController;
import com.nokia.example.engine.PresentationListener;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.media.Player;
import javax.microedition.media.control.GUIControl;
import javax.microedition.media.control.VideoControl;
import javax.microedition.broadcast.ServiceComponent;
import java.util.Vector;

/**
 * Displays the current broadcast stream in a Form
 */
public class VideoFormView extends Form
        implements ViewBase, PresentationListener
{
    // Service guide controller
    private PresentationController controller;
    // Commands
    private Command cmdBack;
    private Command cmdStart;
    private Command cmdStop;

    /**
     * Constructor
     */
    VideoFormView()
    {
        super("Video form");

        cmdBack = new Command("Back", Command.BACK, 1);
        cmdStart = new Command("Start", Command.SCREEN, 1);
        cmdStop = new Command("Stop", Command.SCREEN, 1);

        addCommand(cmdBack);

        controller = PresentationController.getInstance();
    }

    /**
     * From PresentationListener
     */
    public void notifyPresentationEvent(
        int argEventType,
        String argMessage)
    {
        if (argEventType == PresentationListener.EVENT_PRESENTING)
        {
            Player[] varPlayers = controller.getCurrentPlayers();
            // The players should all be realized at this point.
            // Initialize and display them.
            for (int varIndex = 0; varIndex < varPlayers.length; varIndex++)
            {
                Player varPlayer = varPlayers[ varIndex ];
                // Get GUI control object an initialize it to this form
                VideoControl varControl =
                    (VideoControl)varPlayer.getControl("VideoControl");

                if (varControl != null)
                {
                    Item varItem = (Item)varControl.initDisplayMode(
                                    GUIControl.USE_GUI_PRIMITIVE, null);
                    // Show the video in the form
                    append(varItem);
                }
            }
        }
        else if (argEventType == PresentationListener.EVENT_ERROR)
        {
            AlertNote varAlert = new AlertNote("Error",
                                            argMessage,
                                            AlertType.ERROR,
                                            this);
            varAlert.execute();
        }
    }

    /**
     * Activates the view
     */
    public void activate()
    {
        controller.addListener(this);
        controller.start();
        addCommand(cmdStop);
    }

    /**
     * Deactivates the view
     */
    public void deactivate()
    {
        controller.removeListener(this);
        removeCommand(cmdStop);
        removeCommand(cmdStart);

        deleteAll();
        controller.stop();
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable()
    {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param aListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener argListener)
    {
        super.setCommandListener(argListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param argCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable)
    {
        if (argCommand == cmdBack)
        {
            ViewHandler.activateView(ViewHandler.SERVICES_VIEW_ID);
        }
        else if (argCommand == cmdStop)
        {
            removeCommand(cmdStop);
            controller.stop();
            addCommand(cmdStart);
        }
        else if (argCommand == cmdStart)
        {
            removeCommand(cmdStart);
            controller.start();
            addCommand(cmdStop);
        }
    }

}

// End of File
